import { Event, Aggregate, AggregateEncryptionFactory, AggregateProjection, SerializableMap, EventHandlerEncryptionFactory, ReadModelResolvers, ReadModel, SagaEventHandlers, SagaSideEffects, ReadModelQueryResult, Command, ReadModelQuery, CommandResult } from '@resolve-js/core';
import { AggregateTestEnvironment } from './flow/aggregate/make-test-environment';
import { ReadModelTestEnvironment } from './flow/read-model/make-test-environment';
import { SagaTestEnvironment } from './flow/saga/make-test-environment';
export declare type OmitFirstArgument<F> = F extends (first: any, ...args: infer Params) => infer Result ? (...args: Params) => Result : never;
export declare type TestEvent = Omit<Event, 'timestamp' | 'aggregateId' | 'aggregateVersion'> & {
    timestamp?: number;
    aggregateId?: string;
};
export declare type TestAssertion<TResult> = (resolve: (result: TResult) => void, reject: (error: Error) => void, result: TResult, error: any, negated: boolean) => void;
export declare type TestAggregate = {
    name: string;
    projection: AggregateProjection;
    commands: Aggregate;
    encryption?: AggregateEncryptionFactory;
};
export declare type TestCommand = {
    name: string;
    payload?: SerializableMap;
};
export declare type CommandTestResult = {
    type: string;
    payload?: SerializableMap;
} | null;
export declare type TestCommandAssertion = TestAssertion<CommandTestResult>;
export declare type GivenEventsContext = {
    events: TestEvent[];
};
export declare type AggregateContext = {
    aggregate: TestAggregate;
    aggregateId?: string;
} & GivenEventsContext;
export declare type CommandContext = {
    command: TestCommand;
    environment: AggregateTestEnvironment;
} & AggregateContext;
export declare type TestReadModel = {
    name: string;
    projection: ReadModel<any>;
    resolvers: ReadModelResolvers<any>;
    encryption?: EventHandlerEncryptionFactory;
    adapter?: any;
};
export declare type TestQuery = {
    resolver: string;
    args?: SerializableMap;
};
export declare type QueryTestResult = ReadModelQueryResult;
export declare type TestQueryAssertion = TestAssertion<QueryTestResult>;
export declare type ReadModelContext = {
    readModel: TestReadModel;
    adapter?: any;
    encryption?: EventHandlerEncryptionFactory;
} & GivenEventsContext;
export declare type QueryContext = {
    query: TestQuery;
    environment: ReadModelTestEnvironment;
} & ReadModelContext;
export declare type TestSaga<TSideEffects = any> = {
    name: string;
    handlers: SagaEventHandlers<any, SagaSideEffects & TSideEffects>;
    sideEffects: TSideEffects;
    encryption?: EventHandlerEncryptionFactory;
    adapter?: any;
};
export declare type TestSagaAssertion = TestAssertion<SagaTestResult>;
export declare type ScheduledCommand = {
    date: number;
    command: Command;
};
export declare type ExecutedSideEffect = [string, ...any[]];
export declare type SagaTestResult = {
    commands: Array<Command>;
    scheduledCommands: Array<ScheduledCommand>;
    sideEffects: Array<ExecutedSideEffect>;
    queries: Array<ReadModelQuery>;
    scheduleCommands: any[];
};
export declare type SagaContext = {
    saga: TestSaga;
    adapter?: any;
    encryption?: EventHandlerEncryptionFactory;
    environment: SagaTestEnvironment;
} & GivenEventsContext;
export declare type MockedCommandImplementation = (command: Command) => CommandResult;
export declare type MockedQueryImplementation = (query: ReadModelQuery) => ReadModelQueryResult;
